#pragma once

#include <string>
#include <queue>
#include <vector>
#include <thread>
#include <mutex>
#include <condition_variable>
#include <functional>

class TaskManager {
public:
    using Job = std::function<void ()>;
    TaskManager();
    ~TaskManager();

    void start();
    void stop();
    void wait() const;
    void queue(const Job& job);
    bool busy() const;

private:
    void loop();

private:
    bool running;
    bool stopping;
    uint32_t maxThreads;
    uint32_t activeThreads;
    std::queue<Job> jobs;
    mutable std::mutex mutex;
    mutable std::condition_variable loopConditional;
    mutable std::condition_variable waitConditional;
    std::vector<std::thread> threads;
};
